@@ -30,6 +30,9 @@
 #include "util-streaming-buffer.h"
 #include "rust.h"
 
+/* Limit till the data would be buffered in current line */
+#define SMTP_LINE_BUFFER_LIMIT 4096
+
 enum {
     SMTP_DECODER_EVENT_INVALID_REPLY,
     SMTP_DECODER_EVENT_UNABLE_TO_MATCH_REPLY_WITH_REQUEST,
@@ -127,10 +130,12 @@ typedef struct SMTPState_ {
     /** length of the line in current_line.  Doesn't include the delimiter */
     int32_t current_line_len;
     uint8_t current_line_delimiter_len;
+    bool current_line_lf_found;
     /* Consumed bytes till current line */
     int32_t consumed;
     /* If rest of the bytes should be discarded in case of long line w/o LF */
-    bool discard_till_lf;
+    bool discard_till_lf_ts;
+    bool discard_till_lf_tc;
 
     /** var to indicate parser state */
     uint8_t parser_state;
