@@ -537,6 +537,11 @@ void DecodeRegisterPerfCounters(DecodeThreadVars *dtv, ThreadVars *tv)
     dtv->counter_null = StatsRegisterCounter("decoder.null", tv);
     dtv->counter_sll = StatsRegisterCounter("decoder.sll", tv);
     dtv->counter_tcp = StatsRegisterCounter("decoder.tcp", tv);
+
+    dtv->counter_tcp_syn = StatsRegisterCounter("tcp.syn", tv);
+    dtv->counter_tcp_synack = StatsRegisterCounter("tcp.synack", tv);
+    dtv->counter_tcp_rst = StatsRegisterCounter("tcp.rst", tv);
+
     dtv->counter_udp = StatsRegisterCounter("decoder.udp", tv);
     dtv->counter_sctp = StatsRegisterCounter("decoder.sctp", tv);
     dtv->counter_icmpv4 = StatsRegisterCounter("decoder.icmpv4", tv);
@@ -733,7 +738,7 @@ inline int PacketSetData(Packet *p, const uint8_t *pktdata, uint32_t pktlen)
 
 const char *PktSrcToString(enum PktSrcEnum pkt_src)
 {
-    const char *pkt_src_str = "<unknown>";
+    const char *pkt_src_str = NULL;
     switch (pkt_src) {
         case PKT_SRC_WIRE:
             pkt_src_str = "wire/pcap";
@@ -772,6 +777,7 @@ const char *PktSrcToString(enum PktSrcEnum pkt_src)
             pkt_src_str = "capture timeout flush";
             break;
     }
+    DEBUG_VALIDATE_BUG_ON(pkt_src_str == NULL);
     return pkt_src_str;
 }
 
