@@ -171,8 +171,7 @@ static void SBBPrintList(StreamingBuffer *sb)
  *
  * [block][gap][block]
  **/
-static void SBBInit(StreamingBuffer *sb,
-                    uint32_t rel_offset, uint32_t data_len)
+static int WARN_UNUSED SBBInit(StreamingBuffer *sb, uint32_t rel_offset, uint32_t data_len)
 {
     DEBUG_VALIDATE_BUG_ON(!RB_EMPTY(&sb->sbb_tree));
     DEBUG_VALIDATE_BUG_ON(sb->buf_offset > sb->stream_offset + rel_offset);
@@ -180,56 +179,59 @@ static void SBBInit(StreamingBuffer *sb,
     /* need to set up 2: existing data block and new data block */
     StreamingBufferBlock *sbb = CALLOC(sb->cfg, 1, sizeof(*sbb));
     if (sbb == NULL) {
-        return;
+        return -1;
     }
     sbb->offset = sb->stream_offset;
     sbb->len = sb->buf_offset;
+    (void)SBB_RB_INSERT(&sb->sbb_tree, sbb);
+    sb->sbb_size = sbb->len;
+    sb->head = sbb;
 
     StreamingBufferBlock *sbb2 = CALLOC(sb->cfg, 1, sizeof(*sbb2));
     if (sbb2 == NULL) {
-        FREE(sb->cfg, sbb, sizeof(*sbb));
-        return;
+        return -1;
     }
     sbb2->offset = sb->stream_offset + rel_offset;
     sbb2->len = data_len;
-
-    sb->head = sbb;
-    sb->sbb_size = sbb->len + sbb2->len;
-    SBB_RB_INSERT(&sb->sbb_tree, sbb);
-    SBB_RB_INSERT(&sb->sbb_tree, sbb2);
-
     SCLogDebug("sbb1 %"PRIu64", len %u, sbb2 %"PRIu64", len %u",
             sbb->offset, sbb->len, sbb2->offset, sbb2->len);
+
+    sb->sbb_size += sbb2->len;
+    if (SBB_RB_INSERT(&sb->sbb_tree, sbb2) != NULL) {
+        FREE(sb->cfg, sbb2, sizeof(*sbb2));
+        return -1;
+    }
+
 #ifdef DEBUG
     SBBPrintList(sb);
 #endif
-    BUG_ON(sbb2->offset < sbb->len);
+    return 0;
 }
 
 /* setup with leading gap
  *
  * [gap][block]
  **/
-static void SBBInitLeadingGap(StreamingBuffer *sb,
-                              uint64_t offset, uint32_t data_len)
+static int WARN_UNUSED SBBInitLeadingGap(StreamingBuffer *sb, uint64_t offset, uint32_t data_len)
 {
     DEBUG_VALIDATE_BUG_ON(!RB_EMPTY(&sb->sbb_tree));
 
     StreamingBufferBlock *sbb = CALLOC(sb->cfg, 1, sizeof(*sbb));
     if (sbb == NULL)
-        return;
+        return -1;
     sbb->offset = offset;
     sbb->len = data_len;
 
     sb->head = sbb;
     sb->sbb_size = sbb->len;
-    SBB_RB_INSERT(&sb->sbb_tree, sbb);
+    (void)SBB_RB_INSERT(&sb->sbb_tree, sbb);
 
     SCLogDebug("sbb %"PRIu64", len %u",
             sbb->offset, sbb->len);
 #ifdef DEBUG
     SBBPrintList(sb);
 #endif
+    return 0;
 }
 
 static inline void ConsolidateFwd(StreamingBuffer *sb,
@@ -389,10 +391,9 @@ static int Insert(StreamingBuffer *sb, struct SBB *tree,
     return 0;
 }
 
-static void SBBUpdate(StreamingBuffer *sb,
-                      uint32_t rel_offset, uint32_t data_len)
+static int SBBUpdate(StreamingBuffer *sb, uint32_t rel_offset, uint32_t data_len)
 {
-    Insert(sb, &sb->sbb_tree, rel_offset, data_len);
+    return Insert(sb, &sb->sbb_tree, rel_offset, data_len);
 }
 
 static void SBBFree(StreamingBuffer *sb)
@@ -648,9 +649,10 @@ int StreamingBufferAppend(StreamingBuffer *sb, StreamingBufferSegment *seg,
     sb->buf_offset += data_len;
 
     if (!RB_EMPTY(&sb->sbb_tree)) {
-        SBBUpdate(sb, rel_offset, data_len);
+        return SBBUpdate(sb, rel_offset, data_len);
+    } else {
+        return 0;
     }
-    return 0;
 }
 
 /**
@@ -687,9 +689,10 @@ int StreamingBufferAppendNoTrack(StreamingBuffer *sb,
     sb->buf_offset += data_len;
 
     if (!RB_EMPTY(&sb->sbb_tree)) {
-        SBBUpdate(sb, rel_offset, data_len);
+        return SBBUpdate(sb, rel_offset, data_len);
+    } else {
+        return 0;
     }
-    return 0;
 }
 
 #define DATA_FITS_AT_OFFSET(sb, len, offset) \
@@ -754,16 +757,19 @@ int StreamingBufferInsertAt(StreamingBuffer *sb, StreamingBufferSegment *seg,
                 // nothing to do
             } else if (sb->buf_offset) {
                 /* existing data, but there is a gap between us */
-                SBBInit(sb, rel_offset, data_len);
+                if (SBBInit(sb, rel_offset, data_len) < 0)
+                    return -1;
             } else {
                 /* gap before data in empty list */
                 SCLogDebug("empty sbb list: invoking SBBInitLeadingGap");
-                SBBInitLeadingGap(sb, offset, data_len);
+                if (SBBInitLeadingGap(sb, offset, data_len) < 0)
+                    return -1;
             }
         }
     } else {
         /* already have blocks, so append new block based on new data */
-        SBBUpdate(sb, rel_offset, data_len);
+        if (SBBUpdate(sb, rel_offset, data_len) < 0)
+            return -1;
     }
 
     if (rel_offset + data_len > sb->buf_offset)
